;-------------------------------------------------------------------------------
;
;  Raining Squares
;         by Sebastian Mihai, 2015
;         http://sebastianmihai.com
;
;-------------------------------------------------------------------------------

	.include "sfr.i"

;-------------------------------------------------------------------------------
; Variables
;-------------------------------------------------------------------------------
player_x = $30
player_y = $31				; player
obstacle_x = $32
obstacle_y = $33
obstacle_size = $34			; obstacle
random_seed = $35			; used to generate random numbers
background_frame = $36
video_frame = $37
;-------------------------------------------------------------------------------
; END Variables
;-------------------------------------------------------------------------------

	.include "intvec.i"
	.include "gamehdr.i"
	
	jmp begin
	.include "utilities.i"
	
begin:
	clr1 ocr,5				; use the 600khz RC clock
	call init_random
	
	mov #0, background_frame
	mov #0, video_frame
	mov #24, player_x
	mov #31, player_y

	call render_player
	call new_obstacle

	; Main game loop
game_loop:
	call delay
	call render_background
	call handle_input
	call render_player
	call update_obstacle
	br game_loop

;-------------------------------------------------------------------------------
; Draw the background, changing frames as needed
;-------------------------------------------------------------------------------
render_background:
	inc video_frame
	ld video_frame
	and #1							; acc := video_frame mod 2
	bz render_background_continue
	inc background_frame			; every other frame advance animation frame

render_background_continue:	
	ld background_frame
	and #3							; acc := background_frame mod 4
	bne #0, render_background_not0
	mov #<background0, trl
	mov #>background0, trh
	br render_background_done
render_background_not0:
	bne #1, render_background_not1
	mov #<background1, trl
	mov #>background1, trh
	br render_background_done
render_background_not1:
	bne #2, render_background_not2
	mov #<background2, trl
	mov #>background2, trh
	br render_background_done
render_background_not2: 	
	mov #<background3, trl 	; store low byte in low byte address register
	mov #>background3, trh 	; store high byte in high byte address register

render_background_done:	
	call draw_full_screen
	ret
	
;-------------------------------------------------------------------------------
; If the player is hit, add a long delay, and then restart game
;-------------------------------------------------------------------------------
check_collision_horizontally:
	clr1 psw, CY						; clear carry flag
	ld player_x
	sub obstacle_x						;
	bp psw, CY, check_collision_done	; if player_x - obstacle_x < 0
										;    then there is no collision
										; (this is checked via the carry flag
										;  after the subtraction)

	clr1 psw, CY
	ld obstacle_x
	add obstacle_size
	dec acc
	sub player_x
	bp psw, CY, check_collision_done	; if obstacle_x + obstacle_size - 1 - player_x < 0
										;    then there is no collision
	mov #100, acc
check_collision_horizontally_pause:
	call delay
	dbnz acc, check_collision_horizontally_pause
	
check_collision_done:
	ret
	
;-------------------------------------------------------------------------------
; Update the position of the obstacle and redraw it
; When the obstacle is at the bottom, it also checks for a collision with 
; the player
;-------------------------------------------------------------------------------
update_obstacle:
	ld obstacle_y
	add obstacle_size
	bne #32, update_obstacle_still_falling	; if it's in the air, keep it
	call check_collision_horizontally		; handle the case when player was hit
	call new_obstacle
	ret	
update_obstacle_still_falling:
	; move obstacle down
	inc obstacle_y
	call render_obstacle
	
	ret

;-------------------------------------------------------------------------------
; Display the obstacle; first erase the old one, then draw the new one
;-------------------------------------------------------------------------------
render_obstacle:
	push c
	push b
	
	ld obstacle_x
	st c
	ld obstacle_y
	st b
	ld obstacle_size
	call draw_square
	
	pop b
	pop c
	ret

;-------------------------------------------------------------------------------
; Read input and update player position accordingly
;-------------------------------------------------------------------------------
handle_input:
	call read_input
	bn acc, BUTTON_A, handle_input
	bn acc, BUTTON_B, handle_input
	bn acc, DPAD_UP, handle_input
	bn acc, DPAD_LEFT, move_player_left
	bn acc, DPAD_RIGHT, move_player_right
	br handle_input_done

move_player_left:
	ld player_x
	be #8, handle_input_done	; if player_x == 0 then do nothing
	dec player_x				; move player to the left
	br handle_input_done
move_player_right:
	ld player_x
	be #39, handle_input_done	; if player_x == 47 then do nothing
	inc player_x				; move player to the right
	br handle_input_done
handle_input_done:
	ret
	
;-------------------------------------------------------------------------------
; Draw player in the new position, by first erasing the old position
;-------------------------------------------------------------------------------
render_player:
	push c
	push b
	
	ld player_x
	st c
	ld player_y
	st b
	call put_pixel			; draw new player location
	
	pop b
	pop c
	ret

;-------------------------------------------------------------------------------
; Create a new falling obstacle
;-------------------------------------------------------------------------------
new_obstacle:
	push acc
	push b
	push c
	
	mov #0, obstacle_y
	
	mov #8, acc
	call generate_random
	add #2
	st obstacle_size		; obstacle_size := 2 + random( 8 )
	
	mov #32, acc
	sub obstacle_size
	call generate_random
	add #8
	st obstacle_x			; obstacle_x := 8 + random( 48 - 16 - obstacle_size )
							; the - 16 is because of the left and right edges
	
	pop c
	pop b
	pop acc
	ret
	
;-------------------------------------------------------------------------------
; Delay to slow down the game
; Isn't it great how the CPU speed on these old platforms doesn't vary? :)
;-------------------------------------------------------------------------------
delay:
	push acc
	
	mov #250, acc
delay_loop:
	push acc
	mov #2, acc
delay_loop_inner:
	push acc
	mul
	pop acc
	dbnz acc, delay_loop_inner
	pop acc
	dbnz acc, delay_loop

	pop acc
	ret

;-------------------------------------------------------------------------------
; Initialize random number generator
;-------------------------------------------------------------------------------
init_random:	
	clr1 psw,1		; Get random seed from current minute and
	ld $1c			; second system variables
	xor $1d
	set1 psw,1
	st random_seed
	ret
	
;-------------------------------------------------------------------------------
; Generate a random number
; Input:
;	acc - upper bound (exclusive)
; Output:
;	acc - random number
;-------------------------------------------------------------------------------
generate_random:
	push b
	push c
	push acc				; push upper bound
	
	ld random_seed
	st b
	mov #$4e,acc
	mov #$6d,c
	mul
	st b
	ld c
	add #$39
	st random_seed
	ld b
	addc #$30
	
	pop b					; pop upper bound into b, which acts like the divisor
	st c					; c := random
	mov #0, acc
	div						; [acc:c] := [acc:c] div b
							; b := [acc:c] mod b
	
	ld b					; we're returning the result in acc
	
	pop c
	pop b
	ret
	
	.include "graphics/background0.i"
	.include "graphics/background1.i"
	.include "graphics/background2.i"
	.include "graphics/background3.i"

	.cnop 0, $200			; pad
